#!/usr/bin/perl -w
#$Id: compute_amfe.pl,v 1.7 2010/07/07 12:53:56 rnl14480 Exp $

use strict;
use lib qw(../lib);
use BAOBAB::Measure;
use BAOBAB::MeasureWrapper;
use BAOBAB::Evaluation;
use BAOBAB::EvaluationWrapper;
use BAOBAB::Precursor;
use BAOBAB::PrecursorWrapper;
use BAOBAB::Wrapper;
use DBI;

use Data::Dumper qw(&Dumper);

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

my $start = shift;
my $limit = shift;
my $extra = shift;
$extra = undef unless $extra;

my $mname = 'amfe';
my $mw    = BAOBAB::MeasureWrapper->new( dbh => $dbh );
my $m     = $mw->read( name => 'amfe' );
die "Cannot find measure $mname\n" unless defined $m;

my %params = ();
if ( defined $start && defined $limit ) {
	%params = ( start => $start, limit => $limit );
}
$params{extra} = $extra if defined $extra;

my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );
my $pi = $pw->iterator(%params);

while ( my $pid = $pi->() ) {
	my $p     = $pw->read( id => $pid, no_cache => 1 );
	my $len   = length( $p->sequence );
	my $mfe   = $p->mfe;
	my $value = 100 * $mfe / $len;

	my $e = BAOBAB::Evaluation->new(
		precursor => $p,
		measure   => $m,
		value     => $value,
		no_cache  => 1
	);
	my $ew = BAOBAB::EvaluationWrapper->new( dbh => $dbh, object => $e );
	$ew->write;
}

$dbh->disconnect;
