#!/usr/bin/perl -w
#$Id: compute_robust_folding.pl,v 1.10 2010/07/07 12:56:05 rnl14480 Exp $

use strict;
use lib qw(../lib);
use BAOBAB::Measure;
use BAOBAB::MeasureWrapper;
use BAOBAB::Evaluation;
use BAOBAB::EvaluationWrapper;
use BAOBAB::Precursor;
use BAOBAB::PrecursorWrapper;
use BAOBAB::Wrapper;
use DBI;

use Data::Dumper qw(&Dumper);

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

my $start = shift;
my $limit = shift;
my $extra = shift;
$extra = undef unless $extra;

my $mw = BAOBAB::MeasureWrapper->new( dbh => $dbh );
my $m = $mw->read( name => 'robust_folding' );

my %params = ();
if ( defined $start && defined $limit ) {
	%params = ( start => $start, limit => $limit );
}
$params{extra} = $extra if defined $extra;

my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );
my $pi = $pw->iterator(%params);

while ( my $pid = $pi->() ) {
	my $p = $pw->read( id => $pid, no_cache => 1 );

	RNA::pf_fold( $p->sequence );
	my $mean_distance = RNA::mean_bp_dist( length( $p->sequence ) );
	RNA::free_pf_arrays;

	my $value = 100 * $mean_distance / length( $p->sequence );
	print "mean_distance = $mean_distance, value = $value, length = "
	  . length( $p->sequence ) . "\n";

	my $e = BAOBAB::Evaluation->new(
		precursor => $p,
		measure   => $m,
		value     => $value,
		no_cache  => 1
	);
	my $ew = BAOBAB::EvaluationWrapper->new( dbh => $dbh, object => $e );

	$ew->write;
}

$dbh->disconnect;
