#!/usr/bin/perl -w
#$Id: compute_robust_mutation.pl,v 1.6 2010/07/07 12:57:01 rnl14480 Exp $

use strict;
use lib qw(../lib);
use BAOBAB::Measure;
use BAOBAB::MeasureWrapper;
use BAOBAB::Evaluation;
use BAOBAB::EvaluationWrapper;
use BAOBAB::Precursor;
use BAOBAB::PrecursorWrapper;
use BAOBAB::Wrapper;
use DBI;

use Data::Dumper qw(&Dumper);

print STDERR "Connecting to the database...";
my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

my $start = shift;
my $limit = shift;
my $extra = shift;
$extra = undef unless $extra;

my $mw = BAOBAB::MeasureWrapper->new( dbh => $dbh );
my $m = $mw->read( name => 'robust_mutation' );

my %params = ();
if ( defined $start && defined $limit ) {
	%params = ( start => $start, limit => $limit );
}
$params{extra} = $extra if defined $extra;

my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );
my $pi = $pw->iterator(%params);

my @ALPHA = qw(a c u g);
while ( my $pid = $pi->() ) {
	my $p = $pw->read( id => $pid, no_cache => 1 );

	my @dist = ();
	my $seq  = lc( $p->sequence );
	my $str  = $p->structure;
	my @seq  = split //, $seq;

	foreach my $i ( 0 .. length($seq) - 1 ) {
		my @new = @seq;
		foreach my $n (@ALPHA) {
			next if $n eq $seq[$i];
			splice( @new, $i, 1, $n );
			my $nseq = join( "", @new );
			my ( $nstr, undef ) = RNA::fold($nseq);
			push @dist, RNA::bp_distance( $str, $nstr );
		}
	}

	@dist = sort { $a <=> $b } @dist;

	my $midpoint        = scalar(@dist) / 2;
	my $median_distance = 0;
	if ( $median_distance && scalar(@dist) % 2 ) {
		$median_distance = $dist[ int($midpoint) ];
	}
	else {
		$median_distance = ( $dist[$midpoint] + $dist[ $midpoint - 1 ] ) / 2;
	}

	my $value = 100 * $median_distance / length( $p->sequence );

	my $e = BAOBAB::Evaluation->new(
		precursor => $p,
		measure   => $m,
		value     => $value,
		no_cache  => 1
	);
	my $ew = BAOBAB::EvaluationWrapper->new( dbh => $dbh, object => $e );

	$ew->write;
}

$dbh->disconnect;
