#!/usr/bin/perl -w
# $Id: update_overlap.pl,v 1.7 2010/07/07 13:21:25 rnl14480 Exp $

use strict;
use lib qw(../lib/);

use BAOBAB::PrecursorWrapper;
use BAOBAB::Precursor;
use BAOBAB::OverlapWrapper;
use BAOBAB::Overlap;
use BAOBAB::Wrapper;
use DBI;
use MirBase::ParseName qw(parse_name);

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

my @organisms = qw(aga ada dmel);
my %accepted;
$accepted{$_} = 1 for @organisms;

my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );
my $it = $pw->iterator( extra => q{category = 'known'} );
my ( @known, $id );
while ( $id = $it->() ) {
	push @known, $pw->read( id => $id );
}

foreach my $k (@known) {
	next unless exists $accepted{ $k->organism->shortname };
	print "Analyzing precursor " . $k->designation . "\n";
	my $cid    = $k->chromosome->id;
	my $strand = $k->strand;
	my $start  = $k->start;
	my $stop   = $k->stop;
	my $extra =
qq{ cid = $cid AND category = 'candidate' AND stop >= $start AND start <= $stop AND strand = '$strand'};
	my $oit = $pw->iterator( extra => $extra );

	while ( $id = $oit->() ) {
		my $p = $pw->read( id => $id );
		my $pc;
		my $lknown = $stop - $start + 1;
		my $lcand  = $p->stop - $p->start + 1;
		my $type;

		if ( $p->start == $start and $p->stop == $stop ) {
			print "\tExact match\n";
			$pc   = 1;
			$type = 'EXACT';
		}
		elsif ( $p->start <= $start and $p->stop >= $stop ) {
			$pc   = $lknown / $lcand;
			$type = 'SUPERSTRING';
			print "\tSuperstring at $pc\n";
		}
		elsif ( $p->start >= $start and $p->stop <= $stop ) {
			$pc   = $lcand / $lknown;
			$type = 'SUBSTRING';
			print "\tSubstring at $pc\n";
		}
		else {
			if ( $p->start < $start ) {
				$pc = ( $p->stop - $start + 1 ) / $lknown;
			}
			else {
				$pc = ( $stop - $p->start + 1 ) / $lknown;
			}
			$type = 'OVERLAP';
			print "\tOverlap at $pc\n";
		}

		my $ov = BAOBAB::Overlap->new(
			known     => $k,
			candidate => $p,
			type      => $type,
			pc        => $pc
		);

		my $ow = BAOBAB::OverlapWrapper->new( dbh => $dbh, object => $ov );
		$ow->write;
	}
}

$dbh->disconnect;

1;
