#!/usr/bin/perl -w

use strict;

# This script generates an R script which in turn produces
# the composite score for each dataset

die "Usage: $0 indir outdir\n" unless scalar(@ARGV) == 2;

my $indir  = shift || ".";
my $outdir = shift || ".";

my @measure = qw(amfe robust_context robust_mutation robust_folding);
my @org     = qw(ada aga dmel);

print qq[setwd("$outdir/")] . "\n";

foreach my $o (@org) {
	my $title = get_title($o);
	my @m     = ();
	foreach my $m (@measure) {
		my $transformation = get_transformation($m);

		print <<EOL;

r_${m} <- read.table("$indir/all_random_${o}_${m}.out")
a_${m} <- read.table("$indir/all_${o}_${m}.out")

id_${m} <- a_${m}\$V2
r_${m} <- r_${m}\$V1
a_${m} <- a_${m}\$V1
F_${m} <- ecdf(r_${m})
a_${m} <- F_${m}(a_${m})
$transformation
EOL
		push @m, qq[${m}];
	}

	my $astring = join( "*", map { "a_$_" } @m );
	my $istring = join( ",", map { "id_$_" } @m );
	my $first = "id_".$m[0];
	print <<EOL;

		for (i in list($istring)){
			if(length(which($first == i)) != length($first)){
				cat("Evaluation measures are not ordered!")
				quit()
			}
		}

		id <- $first
		score_${o} <- $astring
		data_${o} <- data.frame(id=id, score=score_${o})
		write.table(data_${o},file="$outdir/cscore_${o}.out",sep=":",row.names=FALSE,col.names=FALSE)
EOL
}

sub get_transformation {
	my $m = shift;
	my $t;
	my $invert = qq[
		a_${m} <- 1 - a_${m}
	];

	if ( $m eq 'amfe' or $m eq 'mfei' ) {
		$t = $invert;
	}
	elsif ( $m eq 'robust_context' ) {
		$t = qq[];
	}
	elsif ( $m eq 'robust_folding' ) {
		$t = $invert;
	}
	elsif ( $m eq 'robust_mutation' ) {
		$t = $invert;
	}

	return $t;
}

sub get_title {
	my $o = shift;
	my $t;

	if ( $o eq 'aga' ) {
		$t .= q[A. gambiae];
	}
	elsif ( $o eq 'planted_metazoa' ) {
		$t .= q[metazoa];
	}
	elsif ( $o eq 'dmel' ) {
		$t .= q[D. melanogaster];
	}
	elsif ( $o eq 'ada' ) {
		$t .= q[A. darlingi];
	}
	else {
		$t = $o;
	}

	return $t;
}
